package com.caimei365.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.order.mapper.MessagePushMapper;
import com.caimei365.order.service.RemoteCallService;
import com.caimei365.order.utils.CodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 推送服务
 *
 * @author : Charles
 * @date : 2021/7/15
 */
@Slf4j
@Service
public class RemoteCallServiceImpl implements RemoteCallService {
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private MessagePushMapper messagePushMapper;
    @Autowired
    private WebClient.Builder webClientBuilder;

    /**
     * 生成短链接
     *
     * @param length 链接长度
     * @param markId 跳转类型
     * @param url    跳转地址
     */
    @Override
    public String getShortLink(int length, int markId, String url) {
        String shortLink = CodeUtil.generateShortLink(length);
        Integer id = messagePushMapper.findIdByShortLink(shortLink);
        if (id != null && id > 0) {
            getShortLink(length, markId, url);
        }
        messagePushMapper.insertShortLink(markId, shortLink, url);
        return shortLink;
    }

    /**
     * 请求发短信接口
     *
     * @param markId  短信类型
     * @param mobile  手机号
     * @param content 内容
     */
    @Override
    public boolean getSendSms(int markId, String mobile, String content) {
        AtomicBoolean returnValue = new AtomicBoolean(false);
        try {
            //测试环境手机号允许发短信
            List<String> list = new ArrayList<>();
            list.add("15917362709");
            list.add("15814011616");
            list.add("13100721916");
            list.add("15113936829");
            if ("prod".equals(profile) || list.contains(mobile)) {
                if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
                    String regex = "^(1[3-9]\\d{9}$)";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(mobile);
                    if (matcher.matches()) {
                        //提交参数设置
                        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
                        map.add("mobile", mobile);
                        map.add("content", content);
                        // 发送服务间调用POST请求
                        Mono<String> result = webClientBuilder.build()
                                .post()
                                .uri("http://CAIMEI365-CLOUD-TOOLS/tools/sms/send")
                                .contentType(MediaType.APPLICATION_FORM_URLENCODED)
                                .body(BodyInserters.fromFormData(map))
                                .retrieve()
                                .bodyToMono(String.class);
                        result.subscribe(log::info);
                        result.subscribe(jsonStr -> {
                            JSONObject parseObject = JSONObject.parseObject(jsonStr);
                            if (0 == parseObject.getInteger("code")){
                                messagePushMapper.updateSmsSendCount(markId, 1);
                                returnValue.set(true);
                            }
                        });
                    }
                }
            }
        } catch (Exception e) {
            log.error("短信推送异常:", e);
        }
        return returnValue.get();
    }

    /**
     * 微信小程序授权登录（调用user服务，发送服务间调用POST请求）
     *
     * @param code          微信code
     * @param encryptedData 微信加密数据
     * @param iv            微信加密算法的初始向量
     * @param headers       HttpHeaders
     */
    @Override
    public String appletsAuthorization(String code, String encryptedData, String iv, HttpHeaders headers) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        // 获取当前微信小程序的环境
        String referer = headers.getFirst("Referer");
        //提交参数设置
        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
        map.add("code", code);
        map.add("encryptedData", encryptedData);
        map.add("iv", iv);
        // 发送服务间调用POST请求
        Mono<String> result = webClientBuilder.defaultHeader("Referer", referer).build()
                .post()
                .uri("http://CAIMEI365-CLOUD-USER/user/login/auth/applets")
                .contentType(MediaType.APPLICATION_FORM_URLENCODED)
                .body(BodyInserters.fromFormData(map))
                .retrieve()
                .bodyToMono(String.class);
        result.subscribe(log::info);
        result.subscribe(jsonStr -> {
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code")){
                // 登录成功
                resultData.set(parseObject.getString("data"));
            }
        });
        return resultData.get();
    }

    /**
     * 智能获取物流信息
     *
     * @param number      快递单号
     * @param companyCode 快递公司的编码
     * @param mobile      收/寄件人的电话号码(顺丰必填)
     */
    @Override
    public String getLogisticsByNumber(String number, String companyCode, String mobile) {
        AtomicReference<String> resultData = new AtomicReference<>("");
        // 请求参数
        String params = "?number=" + number;
        if (StringUtils.isNotEmpty(companyCode) && StringUtils.isNotEmpty(mobile)) {
            params = params + ("&companyCode=" + companyCode + "&mobile=" + mobile);
        } else if (StringUtils.isNotEmpty(companyCode)) {
            params = params + ("&companyCode=" + companyCode);
        } else if (StringUtils.isNotEmpty(mobile)) {
            params = params + ("&mobile=" + mobile);
        }
        // 发送服务间调用GET请求
        Mono<String> result = webClientBuilder.build()
                .get()
                .uri("http://CAIMEI365-CLOUD-TOOLS/tools/query/logistics" + params)
                .retrieve()
                .bodyToMono(String.class);
        result.subscribe(log::info);
        result.subscribe(jsonStr -> {
            JSONObject parseObject = JSONObject.parseObject(jsonStr);
            if (0 == parseObject.getInteger("code")){
                // 获取物流信息成功
                resultData.set(parseObject.getString("data"));
            }
        });
        return resultData.get();
    }
}
